/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Dr. Katsuhiko Tsujino
 *   C. Greg Hagerty <cgreg@cgreg.com>
 *    items start out closed
 *    allow undefined types to be self described
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function createElement (tagName) {
    return document.createElementNS("http://www.w3.org/1999/xhtml",tagName);
}

// Called to inform the sidebar of the user's language preference
function setLanguage (language) {
}

// Internal helper function to find the UI element for a given annotation
// uri is the URI identifying the annotation
// first=true to find the first element identifying the annotation
//   (which contains the text of the annotated section of the document)
// or first=false to find the second such elemnt
//   (with the text "[Annotation type] created by [author]...")
function findAnnotation (uri, first) {
    var foundFirst = false;
    var root = document.getElementById("annotationTreeChildren");
    var treeitems = root.getElementsByTagName("treeitem");
    for (var ix=0; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == uri) {
            // If we want the first item, or if we have already found the
            // first item, then this is the one we want
            if (first || foundFirst) {
                return treeitems[ix].parentNode;
            // Otherwise...
            } else {
                foundFirst = true;
            }
        }
    }
    return undefined;
}

function load_selected_body (row) {
   var tree = document.getElementById('annotationTree');
   var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
   if (selectedItem &&
       selectedItem.hasAttributeNS (ANNOZILLA_XUL_NS, "annozilla-body") &&
       selectedItem.getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-body") != undefined &&
       selectedItem.getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-body") != 'undefined') {
       loadAnnotationBody (selectedItem.getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-body"));
   }
}

function edit_selected_body () {
   var tree = document.getElementById('annotationTree');
   var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
   if (selectedItem) {
       edit_annotation_body (
           selectedItem.getAttributeNS(ANNOZILLA_XUL_NS,'annozilla-resource')
       );
   }
}

function delete_selected_annotation () {
   var tree = document.getElementById('annotationTree');
   var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
   if (selectedItem) {
       delete_annotation (
           selectedItem.getAttributeNS(ANNOZILLA_XUL_NS,'annozilla-resource')
       );
   }
}

function check_for_replies () {
    var tree = document.getElementById('annotationTree');
    var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
    if (selectedItem) {
        check_replies (
            selectedItem.getAttributeNS(ANNOZILLA_XUL_NS,'annozilla-resource')
        );
    }
}

// Clear the annotations list from the sidebar.
// parentNode: undefined to clear the whole list,
// or a URI to specify the URI of an annotation to clear the replies from
function clearList (parentNode) {
    var children;
    if (parentNode == undefined) {
        children = document.getElementById("annotationTreeChildren");
        // If we are clearing the entire list then also remove the
        // context and double-click options while there is nothing there
        // (because we will add the "No annotations" message while loading)
        try {
            children.removeAttribute ("context");
            children.removeAttribute ("ondblclick");
        } catch (e) {
        }
        while (children.hasChildNodes()) {
            children.removeChild (children.firstChild);
        }
    } else {
        var children = findAnnotation (parentNode, false);
        while (children.childNodes.length > 1) {
            children.removeChild (children.lastChild);
        }
    }
}

function appendMessageToList (message) {
    var children = document.getElementById("annotationTreeChildren");

    var item = document.createElement ("treeitem");
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("label", message);

    row.appendChild (cell);
    item.appendChild (row);
    children.appendChild (item);
}

function setHeader (message) {
    var header = document.getElementById("headerXUL");
    header.setAttribute ("label",message);
}

function loadAnnotationBody (url, resource) {
    document.getElementById('body').loadURI (url);
    /*
    if (server.supports (server.EDIT)) {
        document.getElementById("edit-annotation").setAttribute
            ("oncommand",
             "javascript:edit_annotation_body ('" + resource + "');");
        document.getElementById("edit-annotation").disabled = false;
    }
    */
}

// Called when the user changes the auto-load preference
function autoLoadPrefChanged (autoload) {
}

// Called when the user changes the "load in new window" preference (obsolete)
function loadInNewWindowPrefChanged (loadInNewWindow) {
}

function updateAnnotation(annotation) {
    var url = annotation.annotationURI;
    var children = document.getElementById("annotationTreeChildren");
    var treeitems = children.getElementsByTagName("treeitem");
    for (var ix =0 ; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS,"annozilla-resource") == url) {
	    treeItem = treeitems[ix];
	    treeItem.setAttributeNS (ANNOZILLA_XUL_NS,"annozilla-body", annotation.bodyURI);
	    var cell2 = treeItem.childNodes.item(1).firstChild.firstChild.firstChild;
	    cell2.setAttribute("label", getAnnotationDescription(annotation));
	    break;
	}
    }
}

function addNewAnnotation (annotation, range, inReplyTo) {

    // The root of the annotation tree
    var root = document.getElementById("annotationTreeChildren");
 
    // The node to which we will attach the annotation
    var parent = root;
    if (inReplyTo) {
        // look for parent: the second treeitem element in document order
        // which has annozilla-resource equal to inReplyTo
        parent = findAnnotation (inReplyTo, false);
    }

    var item = document.createElement ("treeitem");
    item.setAttribute ("container", "true");
    //Need to add a preference? CGH prefers having the items start out closed
    // item.setAttribute ("open", "true");
    item.setAttribute ("open", "false");
    item.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-resource",
           annotation.annotationURI);
    item.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-body",
            annotation.bodyURI);
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("crop", "left");
    cell.setAttribute ("context", "annotationPopup");
    cell.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand()");
    cell.setAttribute ("class", "text");

    var annotationText;
    if (range) {
	annotationText = range.toString();
	if (annotationText.length > 50) {
	    annotationText = annotationText.substring(0, 49) + "...";
	}
	annotationText = annotationText.replace (/\n/g,' ');
	annotationText = annotationText.replace (/^\s+/g,'');
    }
    else {
        var bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");
        annotationText = bundle.GetStringFromName ("UNKNOWN_RANGE");
    }

    cell.setAttribute ("label", annotationText);

    row.appendChild (cell);
    item.appendChild (row);

    var children2 = document.createElement ("treechildren");
    var item2 = document.createElement ("treeitem");
    item2.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-resource",
           annotation.annotationURI);
    item2.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-body",
           annotation.bodyURI);
    var row2  = document.createElement ("treerow");
    var cell2 = document.createElement ("treecell");
    cell2.setAttribute ("crop", "left");
    cell2.setAttribute ("context", "annotationPopup");
    cell2.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand()");
    cell2.setAttribute ("class", "indent");
    cell2.setAttribute ("label", getAnnotationDescription(annotation));

    row2.appendChild (cell2);
    item2.appendChild (row2);
    children2.appendChild (item2);
    item.appendChild (children2);

    parent.appendChild (item);

    root.setAttribute ("context", "annotationPopup");
    root.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand()");
}

// Mapping of the fragment ids of annotation type URIs
// to key values in annozilla.properties
const annotationLookups = {
  Comment : "COMMENT_TYPE",
  SeeAlso : "SEE_ALSO_TYPE",
  Question : "QUESTION_TYPE",
  Explanation : "EXPLANATION_TYPE",
  Example : "EXAMPLE_TYPE",
  Change : "CHANGE_TYPE",
  Advice : "ADVICE_TYPE",
  UNKNOWN : "NO_TYPES"
};

function getAnnotationDescription (annotation) {
  annotation = annotation.QueryInterface(Components.interfaces.nsIAnnotation);
  var bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");
  var type = annotation.data.getProperty(Annozilla.RDF_NS, "type");
  if(type) {
    type = type.substring(type.indexOf("#")+1);
    if (type in annotationLookups) {
       type = bundle.GetStringFromName (annotationLookups[type]);
    } else {
        // allow unknown type to be self described
        // type = bundle.GetStringFromName(annotationLookups["UNKNOWN"]);
    }
  } else {
      type = bundle.GetStringFromName(annotationLookups["UNKNOWN"]);
  }

  var creator = annotation.data.getProperty(Annozilla.DC10_NS, "creator");
  if (!creator) {
      creator = bundle.GetStringFromName("NO_CREATORS");
  }

  var created = annotation.data.getProperty(Annozilla.ANNOT_NS, "created");
  if (!created) {
      created = bundle.GetStringFromName("NO_DATE");
  }

  return bundle.formatStringFromName
       ("ANNOTATION_ITEM", [type, creator, created], 3);
}


function addMessage (message, inReplyTo) {

    // The root of the annotation tree
    var root = document.getElementById("annotationTreeChildren");
 
    // The node to which we will attach the annotation
    var parent = root;
    if (inReplyTo != undefined) {
        // look for parent: the second treeitem element in document order
        // which has annozilla-resource equal to inReplyTo
        var first = false;
        var treeitems = root.getElementsByTagName("treeitem");
        for (var ix=0; ix < treeitems.length; ix++) {
            if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == inReplyTo) {
                if (first) {
                    parent = treeitems[ix].parentNode;
                    break;
                } else {
                    first = true;
                }
            }
        }
    }

    var item = document.createElement ("treeitem");
    item.setAttribute ("container", "false");
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("crop", "left");
    cell.setAttribute ("class", "text");
    cell.setAttribute ("label", message);

    row.appendChild (cell);
    item.appendChild (row);

    parent.appendChild (item);

}

function deleteAnnotationFromUI (url) {
    var children = document.getElementById("annotationTreeChildren");
    var treeitems = children.getElementsByTagName("treeitem");
    for (var ix =0 ; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == url) {
           try {
               treeitems[ix].parentNode.removeChild (treeitems[ix]);
           } catch (e) {
           }
        }
    }
    // TODO remove the icon
}

function setUpUI (server) {
    document.getElementById ("edit-annotation-body")
        .setAttribute ("disabled", !server.supports (server.EDIT));
    document.getElementById ("delete-annotation")
        .setAttribute ("disabled", !server.supports (server.DELETE));
    /*
    document.getElementById ("reply-to-annotation")
        .setAttribute ("disabled", true);
    */
}

function operationInProgress (statusmessage) {
    document.getElementById("progress").mode = "undetermined";
    document.getElementById("statusbar-text").
        setAttribute("label", statusmessage);
}

function operationComplete (statusmessage) {
    document.getElementById("progress").mode = "determined";
    document.getElementById("statusbar-text").
        setAttribute("label", statusmessage);
}
