/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Dr. Katsuhiko Tsujino
 *   C. Greg Hagerty
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const ANNOZILLA_XUL_NS = "http://annozilla.mozdev.org/xul/2005";

var gAnnozillaSidebar;

var browserWindowNamespace = window.parent;

var Annozilla = browserWindowNamespace.Annozilla;

function initializeAnnozillaSidebar () {

    if(!browserWindowNamespace.gAnnozilla){
        browserWindowNamespace.initialize();
    }
    browserWindowNamespace.gAnnozilla.setDefaultBodyDestination(
        document.getElementById("body")
    );
    gAnnozillaSidebar = new AnnozillaSidebar(browserWindowNamespace.gAnnozilla);

    // Enable Options (for Firefox) or Preferences (for Seamonkey)
    const FIREFOX_ID = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
    const SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";

    // Would rather do this with feature detection but how?
    var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
        .getService(Components.interfaces.nsIXULAppInfo);
    if(appInfo.ID == FIREFOX_ID) {
        document.getElementById ("annozillaOptions").setAttribute ("style", "display:block");
    } else if(appInfo.ID == SEAMONKEY_ID) {
        document.getElementById ("annozillaPreferences").setAttribute ("style", "display:block");
    }
}

function createElement (tagName) {
    return document.createElementNS("http://www.w3.org/1999/xhtml", tagName);
}

// Called to inform the sidebar of the user's language preference
function setLanguage (language) {
}

// Internal helper function to find the UI element for a given annotation
// uri is the URI identifying the annotation
// first=true to find the first element identifying the annotation
//   (which contains the text of the annotated section of the document)
// or first=false to find the second such elemnt
//   (with the text "[Annotation type] created by [author]...")
function findAnnotation (uri, first) {
    var foundFirst = false;
    var root = document.getElementById("annotationTreeChildren");
    var treeitems = root.getElementsByTagName("treeitem");
    for (var ix=0; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == uri) {
            // If we want the first item, or if we have already found the
            // first item, then this is the one we want
            if (first || foundFirst) {
                return treeitems[ix].parentNode;
            // Otherwise...
            } else {
                foundFirst = true;
            }
        }
    }
    return undefined;
}

function edit_selected_body () {
   var tree = document.getElementById('annotationTree');
   var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
   if (selectedItem) {
       edit_annotation_body (
           selectedItem.getAttributeNS(ANNOZILLA_XUL_NS,'annozilla-resource')
       );
   }
}

function delete_selected_annotation () {
   var tree = document.getElementById('annotationTree');
   var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
   if (selectedItem) {
       delete_annotation (
           selectedItem.getAttributeNS(ANNOZILLA_XUL_NS,'annozilla-resource')
       );
   }
}

function check_for_replies () {
    var tree = document.getElementById('annotationTree');
    var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
    if (selectedItem) {
        check_replies (
            selectedItem.getAttributeNS(ANNOZILLA_XUL_NS,'annozilla-resource')
        );
    }
}

// Clear the annotations list from the sidebar.
// parentNode: undefined to clear the whole list,
// or a URI to specify the URI of an annotation to clear the replies from
function clearList (parentNode) {
    var children;
    if (parentNode == undefined) {
        children = document.getElementById("annotationTreeChildren");
        // If we are clearing the entire list then also remove the
        // context and double-click options while there is nothing there
        // (because we will add the "No annotations" message while loading)
        try {
            children.removeAttribute ("context");
            children.removeAttribute ("ondblclick");
        } catch (e) {
        }
        while (children.hasChildNodes()) {
            children.removeChild (children.firstChild);
        }
    } else {
        var children = findAnnotation (parentNode, false);
        while (children.childNodes.length > 1) {
            children.removeChild (children.lastChild);
        }
    }
}

function appendMessageToList (message) {
    var children = document.getElementById("annotationTreeChildren");

    var item = document.createElement ("treeitem");
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("label", message);

    row.appendChild (cell);
    item.appendChild (row);
    children.appendChild (item);
}

function setHeader (message) {
    var header = document.getElementById("headerXUL");
    header.setAttribute ("label",message);
}

// Called when the user changes the auto-load preference
function autoLoadPrefChanged (autoload) {
}

// Called when the user changes the "load in new window" preference (obsolete)
function loadInNewWindowPrefChanged (loadInNewWindow) {
}

function updateAnnotation(annotation) {
    var url = annotation.annotationURI;
    var children = document.getElementById("annotationTreeChildren");
    var treeitems = children.getElementsByTagName("treeitem");
    for (var ix =0 ; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS,"annozilla-resource") == url) {
	    treeItem = treeitems[ix];
	    treeItem.setAttributeNS (ANNOZILLA_XUL_NS,"annozilla-body", annotation.bodyURI);
	    var cell2 = treeItem.childNodes.item(1).firstChild.firstChild.firstChild;
	    cell2.setAttribute("label", getAnnotationDescription(annotation));
	    break;
	}
    }
}

function addNewAnnotation (annotation, range, inReplyTo) {

    // The root of the annotation tree
    var root = document.getElementById("annotationTreeChildren");
 
    // The node to which we will attach the annotation
    var parent = root;
    if (inReplyTo) {
        // look for parent: the second treeitem element in document order
        // which has annozilla-resource equal to inReplyTo
        parent = findAnnotation (inReplyTo, false);
    }

    var item = document.createElement ("treeitem");
    item.setAttribute ("container", "true");
    //CGH: prefers them closed
    //item.setAttribute ("open", "true");
    item.setAttribute ("open", "false");
    item.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-resource",
           annotation.annotationURI);
    item.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-body",
            annotation.bodyURI);
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("crop", "left");
    cell.setAttribute ("context", "annotationPopup");
    cell.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand()");
    cell.setAttribute ("class", "text");

    var annotationText;
    if (range) {
	annotationText = range.toString();
	if (annotationText.length > 50) {
	    annotationText = annotationText.substring(0, 49) + "...";
	}
	annotationText = annotationText.replace (/\n/g,' ');
	annotationText = annotationText.replace (/^\s+/g,'');
    }
    else {
        var bundle = srGetStrBundle(
            "chrome://annozilla/locale/annozilla.properties"
        );
        annotationText = bundle.GetStringFromName ("UNKNOWN_RANGE");
    }

    cell.setAttribute ("label", annotationText);

    row.appendChild (cell);
    item.appendChild (row);

    var children2 = document.createElement ("treechildren");
    var item2 = document.createElement ("treeitem");
    item2.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-resource",
           annotation.annotationURI);
    item2.setAttributeNS
          (ANNOZILLA_XUL_NS,
           "annozilla-body",
           annotation.bodyURI);
    var row2  = document.createElement ("treerow");
    var cell2 = document.createElement ("treecell");
    cell2.setAttribute ("crop", "left");
    cell2.setAttribute ("context", "annotationPopup");
    cell2.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand()");
    cell2.setAttribute ("class", "indent");
    cell2.setAttribute ("label", getAnnotationDescription(annotation));

    row2.appendChild (cell2);
    item2.appendChild (row2);
    children2.appendChild (item2);
    item.appendChild (children2);

    parent.appendChild (item);

    root.setAttribute ("context", "annotationPopup");
    root.setAttribute ("ondblclick", "gAnnozillaSidebar.loadBodyOnCommand()");
}

// Mapping of the fragment ids of annotation type URIs
// to key values in annozilla.properties
const annotationLookups = {
  Comment : "COMMENT_TYPE",
  SeeAlso : "SEE_ALSO_TYPE",
  Question : "QUESTION_TYPE",
  Explanation : "EXPLANATION_TYPE",
  Example : "EXAMPLE_TYPE",
  Change : "CHANGE_TYPE",
  Advice : "ADVICE_TYPE",
  UNKNOWN : "NO_TYPES"
};

function getAnnotationDescription (annotation) {
  annotation = annotation.QueryInterface(Components.interfaces.nsIAnnotation);
  var bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");
  var type = annotation.data.getProperty(Annozilla.RDF_NS, "type");
  if(type) {
    type = type.substring(type.indexOf("#")+1);
    if (type in annotationLookups) {
       type = bundle.GetStringFromName (annotationLookups[type]);
    } else {
        //type = bundle.GetStringFromName(annotationLookups["UNKNOWN"]);
        //allow undefined type to be self described (i.e. show whatever it is)
    }
  } else {
      type = bundle.GetStringFromName(annotationLookups["UNKNOWN"]);
  }

  var creator = annotation.data.getProperty(Annozilla.DC10_NS, "creator");
  if (!creator) {
      creator = bundle.GetStringFromName("NO_CREATORS");
  }

  var created = annotation.data.getProperty(Annozilla.ANNOT_NS, "created");
  if (!created) {
      created = bundle.GetStringFromName("NO_DATE");
  }

  return bundle.formatStringFromName
       ("ANNOTATION_ITEM", [type, creator, created], 3);
}


function addMessage (message, inReplyTo) {

    // The root of the annotation tree
    var root = document.getElementById("annotationTreeChildren");
 
    // The node to which we will attach the annotation
    var parent = root;
    if (inReplyTo != undefined) {
        // look for parent: the second treeitem element in document order
        // which has annozilla-resource equal to inReplyTo
        var first = false;
        var treeitems = root.getElementsByTagName("treeitem");
        for (var ix=0; ix < treeitems.length; ix++) {
            if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == inReplyTo) {
                if (first) {
                    parent = treeitems[ix].parentNode;
                    break;
                } else {
                    first = true;
                }
            }
        }
    }

    var item = document.createElement ("treeitem");
    item.setAttribute ("container", "false");
    var row  = document.createElement ("treerow");
    var cell = document.createElement ("treecell");
    cell.setAttribute ("crop", "left");
    cell.setAttribute ("class", "text");
    cell.setAttribute ("label", message);

    row.appendChild (cell);
    item.appendChild (row);

    parent.appendChild (item);

}

function deleteAnnotationFromUI (url) {
    var children = document.getElementById("annotationTreeChildren");
    var treeitems = children.getElementsByTagName("treeitem");
    for (var ix =0 ; ix < treeitems.length; ix++) {
        if (treeitems[ix].getAttributeNS (ANNOZILLA_XUL_NS, "annozilla-resource") == url) {
           try {
               treeitems[ix].parentNode.removeChild (treeitems[ix]);
           } catch (e) {
           }
        }
    }
    // TODO remove the icon
}

function setUpUI (server) {
    document.getElementById ("edit-annotation-body")
        .setAttribute ("disabled", !server.supports (server.EDIT));
    document.getElementById ("delete-annotation")
        .setAttribute ("disabled", !server.supports (server.DELETE));
    /*
    document.getElementById ("reply-to-annotation")
        .setAttribute ("disabled", true);
    */
}

function operationInProgress (statusmessage) {
    document.getElementById("progress").mode = "undetermined";
    document.getElementById("statusbar-text").
        setAttribute("label", statusmessage);
}

function operationComplete (statusmessage) {
    document.getElementById("progress").mode = "determined";
    document.getElementById("statusbar-text").
        setAttribute("label", statusmessage);
}

function AnnozillaSidebar(annozilla0) {

    this.bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");

    this.annozilla = annozilla0;

    // add our observer
    this.annozilla.app.addObserver(this);

    //let annozilla resolve and add annotations in order
    annozilla0.sidebar = this;
    dump("Annozilla, meet Sidebar\n");

    var that = this;

    // add a listener for when the window closes
    window.addEventListener("unload",
    			    function(e) { that.unload(); },
			    false);
}

AnnozillaSidebar.prototype = {
    bundle: undefined,
    serversResponded: undefined
};

// From the original (Annozilla 0.6.5.2) AnnozillaSidebar.
AnnozillaSidebar.prototype.clearList = function() {
    clearList();
}

AnnozillaSidebar.prototype.operationInProgress = function(bundleStr) {
    operationInProgress(this.bundle.GetStringFromName(bundleStr));
}

AnnozillaSidebar.prototype.operationComplete = function(bundleStr) {
    operationComplete(this.bundle.GetStringFromName(bundleStr));
}

AnnozillaSidebar.prototype.setHeader = function(message) {
    setHeader(message);
}


AnnozillaSidebar.prototype.addAnnotations = function(annotations) {
    /* XXX Asynch load change originally in annozilla.js;
       moved to here - is this right? MJW 
    if (this.annozilla.prefs.getPref('loadAnnotationsAsynch')) { 
        this.addAnnotationsAsynch(annotations);
    } else {
    */
        for (uri in annotations) {
            // XXX don't add broken annotations
            if (annotations[uri].annot.data) {
                addNewAnnotation(
                    annotations[uri].annot,
                    annotations[uri].range,
                    null
                );
            }
        }

}

AnnozillaSidebar.prototype.getSelectedAnnotationURI = function() {
    var tree = document.getElementById('annotationTree');
    var selectedItem = tree.contentView.getItemAtIndex(tree.currentIndex);
    if (selectedItem) {
        return selectedItem.getAttributeNS(ANNOZILLA_XUL_NS, "annozilla-resource");
    }
    else {
	return null;
    }
}

AnnozillaSidebar.prototype.updateAnnotation = function(annotation) {
    updateAnnotation(annotation);
}

AnnozillaSidebar.prototype.deleteAnnotation = function(url) {
    deleteAnnotationFromUI(url);
}

// End from the original (Annozilla 0.6.5.2) AnnozillaSidebar.

AnnozillaSidebar.prototype.unload = function() {
    dump("Sidebar: unloading\n");
    // remove observer
    this.annozilla.app.removeObserver(this);
    //doesn't work Annozilla.sidebar = null;
    browserWindowNamespace.gAnnozilla.byeByeSidebar();
}
    

AnnozillaSidebar.prototype.QueryInterface = function(iid) {  
    if (! (iid.equals(Components.interfaces.nsIAnnotationsObserver) ||
	   iid.equals(Components.interfaces.nsISupports) ) ) {
	       throw Components.results.NS_ERROR_NO_INTERFACE;
	   }
    return this;
}

/**
 * Responds to a user pressing the load button
 * by loading annotations for the current page.
 * Signifies that from here on out, we should
 * pay attention to load callbacks instead of autoLoad
 * callbacks.
 */   
AnnozillaSidebar.prototype.userLoadOnCommand = function() {
    this.serversResponded = 0;

    var serverCount = this.annozilla.currentServerCount();
    if (serverCount == 0) {
        this.clearList();
        this.setHeader (this.bundle.GetStringFromName("NO_SERVERS"));
        return false;
    }

    this.clearList();
    this.setHeader(
        this.bundle.formatStringFromName("USERLOADING_COUNT",[0,serverCount],2)
    );
    this.operationInProgress("LOADING");

    // do the load
    this.annozilla.userLoadOnCommand();

    return true;
}

AnnozillaSidebar.prototype.postOnCommand = function() {
    this.annozilla.postOnCommand();
}

AnnozillaSidebar.prototype.updateOnCommand = function() {
    // get the selected item
    var annotationURI = this.getSelectedAnnotationURI();

    this.annozilla.updateOnCommand(annotationURI);
}

AnnozillaSidebar.prototype.deleteOnCommand = function() {
    // get the selected item
    var annotationURI = this.getSelectedAnnotationURI();

    var tree = document.getElementById('annotationTree');
    //need to grab index before delete to select next item
    var index = tree.currentIndex;

    this.annozilla.deleteOnCommand(annotationURI);

    if (index > tree.view.rowCount) {
        // Deleted the last item, so select the new last item
        tree.view.selection.select(tree.view.rowCount-1); 
    } else {
        tree.view.selection.select(index); 
    }
}

AnnozillaSidebar.prototype.loadBodyOnCommand = function() {
    // get the selected item
    var annotationURI = this.getSelectedAnnotationURI();

    this.annozilla.loadBodyOnCommand(annotationURI);
}
  

/**
 * Called when a page begins loading in the 
 * browser if the application has 
 * autoLoad turned on.  
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url loading
 */
AnnozillaSidebar.prototype.autoLoadOnBeginPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
	    //return; // commented out because it can abandon legitimate autoload operations
    }

    // TODO should this be in the service?
    if(!this.annozilla.autoLoadAllowSite(url)) return false;

    this.serversResponded = 0;

    var serverCount = this.annozilla.currentServerCount();
    if (serverCount == 0) {
        this.clearList();
        this.setHeader(this.bundle.GetStringFromName("NO_SERVERS"));
        return false;
    }

    this.clearList();
    this.setHeader(
        this.bundle.formatStringFromName (
            "AUTOLOADING_COUNT", [0, serverCount], 2));
    this.operationInProgress("AUTOLOADING");

    return true;
}

/**
 * Called when a page successfully finishes loading.
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url finished loading
 * @param annotations nsISimpleEnumeration of annotations for url
 */
AnnozillaSidebar.prototype.autoLoadOnSuccessPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
        return;
    }

    // set the taskbar
    this._setTaskBar();

    /* Let Annozilla add them in order and call addAnnotation
    // Resolve all outstanding annotations.
    this.annozilla.resolveAnnotations();

    // add these annotations to the sidebar

    this.addAnnotations(this.annozilla.annotations);
    */

    var serverCount=this.annozilla.currentServerCount();

    if (this.serversResponded && serverCount) {
        this.setHeader(  this.bundle.formatStringFromName(
            "AUTOLOADING_COUNT",
            [this.serversResponded, serverCount],
            2
        ));
    }
}

/**
 * Called when a page that was loading is
 * unable to load (or is cancelled by the user).
 *
 * Note: only called if autoLoad is enabled.
 *
 * @param url string of url aborted
 * @param message string error message
 */
AnnozillaSidebar.prototype.autoLoadOnFailPageLoad = function(url, win) {

    // ensure correct window
    if (win != window.content) {
	return;
    }

    // add the annotations to the sidebar ?
    this.addAnnotations(this.annozilla.annotations);

    var serverCount = this.annozilla.currentServerCount();
    // document failed page load
    if (this.serversResponded == serverCount) {
        this.operationComplete("PAGE_LOAD_ERROR");
    }
    else {
        this.operationInProgress("PAGE_LOAD_ERROR");
    }
}


/**
 * Success callback function to return the results for an
 * automatic annotations load (autoLoad).
 * This function returns the annotations for a single server, designated by serverURI.  
 * It will be called once for each server that successfully responds.
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri that the annotations annotate
 * @param annotations nsISimpleEnumerator of nsIAnnotations for annotatedURI that
 *                    were on serverURI annotations server
 */
AnnozillaSidebar.prototype.autoLoadAnnotationsForServer = function(
    serverURI,
    win,
    annotatedURI,
    annotationsLength,
    annotations
){

    // ensure correct window
    if (win != window.content) {
	return;
    }

    // if there is a user load in progress, ignore these annotations
    if (this.annozilla.userLoadInProgress()) {
	dump("ignoring autoLoad annotations from " + serverURI + " about " + annotatedURI + " since userLoad in progress\n");
	return;
    }

    this.serversResponded += 1;

    this._processAnnotationsWithAutoLoadInfo(
        this.annozilla.currentURIState(),
        annotations
    );

    this._setTaskBar();

    var serverCount = this.annozilla.currentServerCount();
    if (this.serversResponded && serverCount) {
        this.setHeader(
            this.bundle.formatStringFromName (
                "AUTOLOADING_COUNT", [this.serversResponded, serverCount], 2));
    }
}

/**
 * Callback function to indicate failure for an automatic
 * annotations load (autoLoad).
 * This function will be called once for each annotations server that
 * fails in the autoloading process. 
 *
 * @param serverURI string uri of the annotations server that failed
 * @param annotatedURI string uri for which annotations were requested
 * @param message string error message
 */ 
AnnozillaSidebar.prototype.autoLoadFailureForServer = function(
    serverURI,
    win,
    annotatedURI,
    message
){

    // ensure correct window
    if (win != window.content) {
	return;
    }

    this.serversResponded += 1;
}


/**
 * Success callback function to return the results for a loadAnnotations request
 *  - OR - an automatic autoLoad request.  This function returns the 
 * annotations for one server, designated by serverURI.  
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri that the annotations annotate
 * @param annotations nsISimpleEnumerator of nsIAnnotations for annotatedURI
 *                    that were on serverURI annotations server
 */
AnnozillaSidebar.prototype.loadAnnotationsOnSuccess = function(serverURI, annotatedURI, annotationsLength, annotations) {
    // If we're autoLoading, ignore these annotations.
    if(!this.annozilla.userLoadInProgress()){
        // this.annozilla prints an error for this.
        return;
    }

    // If we've moved on to a new page, ignore these annotations.
    if(annotatedURI != this.annozilla.currentURI()) {
        // this.annozilla prints an error for this.
        return;
    }

    var serverCount=this.annozilla.currentServerCount(

    );
    try {
        this.serversResponded += 1;

        // If autoLoad is enabled, we can benefit from its alerts about when
        // the page is fully loaded.
        if(this.annozilla.isAutoLoadEnabled()) {
            this._processAnnotationsWithAutoLoadInfo(
                this.annozilla.currentURIState(),
                annotations
            );
 
            this.setHeader(this.bundle.formatStringFromName(
                "USERLOADING_COUNT",
                [ this.serversResponded, serverCount ],
                2
            ));
            this._setTaskBar();
        } else {
            // Otherwise, we have to wing it. for now we'll assume that the page
            // is loaded, since there's no way that I know to check whether the
            // DOM is fully constructed.

            var currentAnnotationsBatch = {};

            for(var i = 0; i < annotations.length; i++) {
                var current = annotations[i];

                // testing purposes
                if (! current.data) {
                    this._debugBadAnnotation(current);
                    continue;
                }

                var range = this.annozilla.resolveAnnotation(current);

                // Add to the current batch.
                AnnozillaSidebar._addAnnotationToContainer(
                    current,
                    range,
                    currentAnnotationsBatch
                );
            }

            // Add this batch to the sidebar.
            this.addAnnotations(currentAnnotationsBatch);

            if (this.serversResponded == serverCount) {
                this.operationComplete("LOAD_OK");
                var menuitem = window.document.getElementById('stop-load');
                menuitem.hidden = 1;
            }
            this.setHeader( this.bundle.formatStringFromName(
                "USERLOADING_COUNT",
                [this.serversResponded, serverCount],
                2
            ));
        }

    } catch (e) {
        var consoleService=Components.classes['@mozilla.org/consoleservice;1'];
        var console = consoleService.getService(
            Components.interfaces.nsIConsoleService
        );
        console.logStringMessage("Annozilla: error loading annotations:");
        Components.utils.reportError (e);
        this.operationComplete("LOAD_ERROR");
    }
}

/**
 * Callback function to indicate failure for a loadAnnotations request
 *  - OR - an automatic autoLoad request.  This function indicates failure
 * for one particular annotations server, designated by serverURI.
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri for which annotations were requested
 * @param message string error message
 */
AnnozillaSidebar.prototype.loadAnnotationsOnFailure = function(
    serverURI,
    annotatedURI,
    message
){
    // this.annozilla prints an error for this.
}

/**
 * Called when an annotation post request succeeds.
 *
 * @param newAnnotation nsIAnnotation the newly created annotation
 */
AnnozillaSidebar.prototype.postAnnotationOnSuccess = function(newAnnotation) {
    // Could resolve redundantly below and add, 
    //  but let's just let Annozilla call us
}
    /*
    // if this annotation applies to a page we're no longer viewing
    var uri = newAnnotation.data.getProperty(Annozilla.ANNOT_NS, "annotates");
    if (uri != this.annozilla.currentURI()) {
        // this.annozilla prints an error for this.
        return;
    }

    var currentAnnotationsBatch = {};

    var range = this.annozilla.resolveAnnotation(newAnnotation);

    // add to the current batch
    AnnozillaSidebar._addAnnotationToContainer(
        newAnnotation,
        range,
        currentAnnotationsBatch
    );

    // add this batch to the sidebar
    this.addAnnotations(currentAnnotationsBatch);

//  var firstItem=document.getElementById("annotationTreeChildren").firstChild;
//  if(firstItem && firstItem.????){
//      // Make the top status bar show a message to indicate that the only
//      // annotations shown are newly posted ones.
//      this.setHeader("Annotations just posted (none from servers):");
//  }
}
    */

/**
 * Called when an annotation post fails.
 *
 * @param message string error message
 */
AnnozillaSidebar.prototype.postAnnotationOnFailure = function(message) {}



/**
 * Called when an annotation update succeeds.
 *
 * @param updatedAnnotation nsIAnnotation that has been updated on server
 */
AnnozillaSidebar.prototype.updateAnnotationOnSuccess = function(
    updatedAnnotation
){
    var uri = updatedAnnotation.annotationURI;
    if (findAnnotation(uri, true) != undefined) {
        this.updateAnnotation(updatedAnnotation);
    }
}

/**
 * Called when an annotation update fails.
 *
 * @param url string url of the annotation that could not be updated
 * @param message string error message
 */
AnnozillaSidebar.prototype.updateAnnotationOnFailure = function(url, message) {}



/**
 * Called when an annotation is successfully deleted.
 *
 * @param url string url of deleted annotation
 */
AnnozillaSidebar.prototype.deleteAnnotationOnSuccess = function(url) {
    this.deleteAnnotation(url);
}

/**
 * Called when an annotation delete fails.
 *
 * @param url string url of the annotation that could not be deleted
 * @param message string error message
 */
AnnozillaSidebar.prototype.deleteAnnotationOnFailure = function(url, message) {
    // this.annozilla prints an error message.
}

/**
 * Called when the replies to an annotation have been successfully
 * loaded.
 * 
 * @param annotation nsIAnnotation for which replies were requested
 */
AnnozillaSidebar.prototype.getRepliesOnSuccess = function(annotation, repliesCount, replies) {}

/**
 * Called when getting the replies to an annotation fails.
 *
 * @param annotation nsIAnnotation for which replies were requested
 * @param message string error message
 */
AnnozillaSidebar.prototype.getRepliesOnFailure = function(annotation, message) {}

/**
 * Called an annotation has successfully been replied to.
 * 
 * @param original nsIAnnotation that existed originally
 * @param reply nsIAnnotation reply created
 */
AnnozillaSidebar.prototype.replyToAnnotationOnSuccess = function(original, reply) {}

/**
 * Called when replying to an annotation fails.
 *
 * @param original nsIAnnotation to which we attempted to reply
 * @param message string error message
 */
AnnozillaSidebar.prototype.replyToAnnotationOnFailure = function(original, message) {}

/**
 * Called after a loadAnnotationBody request
 * when the body text of the annotation has been successfully loaded.
 *
 * @param annotation nsIAnnotation with updated body text
 * @param bodyText string text of the annotation's body
 */
AnnozillaSidebar.prototype.loadAnnotationBodyOnSuccess = function(annotation, bodyText) {}

/**
 * Called after a loadAnnotationBody request when
 * the body text fails to load.
 *
 * @param annotation nsIAnnotation that body was requested for (unmodified)
 * @param message string error message
 */
AnnozillaSidebar.prototype.loadAnnotationBodyOnFailure = function(annotation, message) {}

AnnozillaSidebar._addAnnotationToContainer=Annozilla._addAnnotationToContainer;

AnnozillaSidebar.prototype._setTaskBar = function() {
  
    // if all servers are responded, update taskbar
    var serverCount = this.annozilla.currentServerCount();
    if (this.serversResponded == serverCount) {
        var currentURIState = this.annozilla.currentURIState();
        if (currentURIState == Annozilla.PAGE_LOADING) {
            this.operationInProgress("LOAD_WAIT");
        }
        else if (currentURIState == Annozilla.PAGE_FAILED) {
            this.operationComplete("PAGE_LOAD_ERROR");
        }
        else {
            this.operationComplete("LOAD_OK");
            var menuitem = window.document.getElementById('stop-load');
            menuitem.hidden = 1;
        }
    }
}

AnnozillaSidebar.prototype._debugBadAnnotation = Annozilla._debugBadAnnotation;

AnnozillaSidebar.prototype._processAnnotationsWithAutoLoadInfo = function(
    state,
    annotations
){
    /*
    if (state == Annozilla.PAGE_LOADING) {
        return;
    }
    var currentAnnotationsBatch = {};

    for(var i = 0; i < annotations.length; i++) {
        var current = annotations[i];

        if (! current.data) {
            this._debugBadAnnotation(current);
            continue;
        }

        if(state == Annozilla.PAGE_COMPLETED){
            var range = this.annozilla.resolveAnnotation(current);

            // Add to the current batch.
            AnnozillaSidebar._addAnnotationToContainer(
                current,
                range,
                currentAnnotationsBatch
            );
        }else{
            // Add to the current batch.
            AnnozillaSidebar._addAnnotationToContainer(
                current,
                null,
                currentAnnotationsBatch
            );
        }
    }
    // Add this batch to the sidebar.
    this.addAnnotations(currentAnnotationsBatch);
    */

}


AnnozillaSidebar.prototype.addAnnotation = function(annot, range) {
  addNewAnnotation(annot, range, null);
}

AnnozillaSidebar.prototype.scrollIntoViewOnCommand = function() {
  var annotationURI = this.getSelectedAnnotationURI();
  if (! (annotationURI && this.annozilla.annotations[annotationURI]) ) {
    window.alert("Unable to scroll to the selected annotation due to an internal error. " +
		 " Try reloading the annotations for the page.");
    return;
  }
  var annotation = this.annozilla.annotations[annotationURI].annot;
  var range = xptrResolveAnnotation(annotation, window._content.document);
  if (range) {
      var startNode = range.startContainer;
      while (startNode.nodeType!=Node.ELEMENT_NODE) 
          startNode = startNode.parentNode;
     startNode.scrollIntoView();
  }
}

// Open Firefox options. From extensions.js
AnnozillaSidebar.prototype.openOptions = function () {

    var optionsURL = "chrome://annozilla/content/prefs/firefox/redirector.xul";
    var windows = Components.classes['@mozilla.org/appshell/window-mediator;1']
        .getService(Components.interfaces.nsIWindowMediator)
        .getEnumerator(null);
    while (windows.hasMoreElements()) {
        var win = windows.getNext();
        if (win.document.documentURI == optionsURL) {
            win.focus();
            return;
        }
    }

    var features;
    try {
        var instantApply =
            Components.classes["@mozilla.org/preferences-service;1"].
            getService(Components.interfaces.nsIPrefBranch).
            getBoolPref("browser.preferences.instantApply");
        features = "chrome,titlebar,toolbar,centerscreen" + (instantApply ? ",dialog=no" : ",modal");
    }
    catch (e) {
        features = "chrome,titlebar,toolbar,centerscreen,modal";
    }
    window.openDialog(optionsURL, "", features);

}

// From utilityOverlay.js goPreferences. Open Mozilla SeaMonkey preferences
AnnozillaSidebar.prototype.openPreferences = function () {

    var containerID = "annozillaItem";
    var paneURL = "chrome://annozilla/content/prefs/pref-annozilla-viewing.xul";
    var itemID = "annozillaViewing";

    var resizable;
    var pref = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch);
    try {
        // We are resizable ONLY if in box debugging mode, because in
        // this special debug mode it is often impossible to see the 
        // content of the debug panel in order to disable debug mode.
        resizable = pref.getBoolPref("xul.debug.box");
    }
    catch (e) {
        resizable = false;
    }

    //check for an existing pref window and focus it; it's not application modal
    const kWindowMediatorContractID = "@mozilla.org/appshell/window-mediator;1";
    const kWindowMediatorIID = Components.interfaces.nsIWindowMediator;
    const kWindowMediator = Components.classes[kWindowMediatorContractID].getService(kWindowMediatorIID);
    var lastPrefWindow = kWindowMediator.getMostRecentWindow("mozilla:preferences");
    if (lastPrefWindow)
        lastPrefWindow.focus();
    else {
        var resizability = resizable ? "yes" : "no";
        var features = "chrome,titlebar,resizable=" + resizability;
        openDialog("chrome://communicator/content/pref/pref.xul","PrefWindow", 
                features, paneURL, containerID, itemID);
    }

}
